unit TBar;

interface

uses Messages, Windows, SysUtils, Classes, Graphics, Menus, Controls, Forms,
   VCLControl, ImgList, CommCtrl, StdCtrls, ExtCtrls;

type
 TabTBar = class;
 TButt = class;
 TButts = class;

  TBEdgeBorder = (ebLeft, ebTop, ebRight, ebBottom);
  TBEdgeBorders = set of TBEdgeBorder;
  TBEdgeStyle = (esNone, esRaised, esLowered);
  TButtstyle = (ttButton, ttToggle, ttDivider, ttDropDown, ttGroup,
                  ttSeparator, ttComboBox, ttEditor);

  TImageIndex = type Integer;

  TButtDownEvent = procedure(ButtDown: Boolean) of object;

  TTPopupMenu = Class(TPopupMenu)
   private
    FReturnMsg: TControl;
   Public
    Procedure PopUpMsg(X, Y: Integer; ReturnMsg: TControl);
    procedure MenuChanged(Sender: TObject; Source: TMenuItem; Rebuild: Boolean); override;
     //CM_MENUCHANGED message sent to FReturnMsg
  end;

 TabGBut = Class(TGraphicControl)
  private
    FBBut: TButt;
    FBorderWidth: Integer;
    FEdgeOuter: TBEdgeStyle;
    FBButStyle: TButtstyle;
    FDown: Boolean;
    FEnabled: Boolean;
    FDownButDown: Boolean;
    FImageIndex: TImageIndex;
    FDropdownMenu: TTPopupMenu;
    FCheck: Boolean;
    SetButtClick: TButtDownEvent;
    FOnMenuDown: TButtDownEvent;
    procedure SetEdgeOuter(const Value: TBEdgeStyle);
    procedure SetButStyle(const Value: TButtstyle);
    function IsCheckedStored: Boolean;
    procedure SetDown(const Value: Boolean);
    procedure SetImageIndex(const Value: TImageIndex);
    procedure SetDropdownMenu(const Value: TTPopupMenu);
    function GetVisible: Boolean;
    procedure SetVisible(const Value: Boolean);
   protected
    procedure SetEnabled(Value: Boolean); override;
    procedure NCPaint(DC: HDC); virtual;
    procedure WMNCCalcSize(var Message: TWMNCCalcSize); message WM_NCCALCSIZE;
    procedure WMNCPaint(var Message: TMessage); message WM_NCPAINT;
    procedure CMBorderChanged(var Message: TMessage); message CM_BORDERCHANGED;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMMENUCHANGED(var Message); message CM_MENUCHANGED;
    procedure CM_LOSTFOCUS(var Message); message CM_LOSTFOCUS;
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   public
    property EdgeOuter: TBEdgeStyle read FEdgeOuter write SetEdgeOuter default esRaised;
   published
    property Canvas;
    property Visible: Boolean read GetVisible write SetVisible;
    property Enabled: Boolean read FEnabled write SetEnabled;
    property Down: Boolean read FDown write SetDown stored IsCheckedStored default False;
    property Style: TButtstyle read FBButStyle write SetButStyle default ttButton;
    property ImageIndex: TImageIndex read FImageIndex write SetImageIndex;
    property Name;
    property ButtClick: TButtDownEvent read SetButtClick write SetButtClick;
    property MenuClick: TButtDownEvent read FOnMenuDown write FOnMenuDown;
end;

 TGBut = Class(TabGBut)
   public
    constructor Create(AOwner: TComponent); override;
 end;

 TButt = class(TCollectionItem)
   private
    FGBut: TGBut;
    FBButs: TButts;
    FCombo: TComboBox;
    FEdit: TEdit;
    FGName: String;
    FText: String;
    FHint: String;
    SetButtClick: TButtDownEvent;
    FOnMenuDown: TButtDownEvent;
    FOnMenuUp: TButtDownEvent;
    procedure SetButStyle(const Value: TButtstyle);
    function GetButStyle: TButtstyle;
    procedure SetName(const Value: String);
    procedure SetEnabled(const Value: Boolean);
    procedure SetImageIndex(const Value: TImageIndex);
    procedure SetHint(const Value: String);
    procedure SetText(const Value: String);
    function GetDropDown: TStrings;
    procedure SetDropDown(const Value: TStrings);
    procedure SetDropdownMenu(const Value: TTPopupMenu);
    function GetDropdownMenu: TTPopupMenu;
    function GetLeft: Integer;
    function GetWidth: Integer;
    procedure SetLeft(const Value: Integer);
    function GetImageIndex: TImageIndex;
    function GetEnabled: Boolean;
    function GetVisible: Boolean;
    procedure SetVisible(const Value: Boolean);
    function GetComboStyle: TComboBoxStyle;
    procedure SetComboStyle(const Value: TComboBoxStyle);
    function GetComboCount: Integer;
    procedure SetWidth(const Value: Integer);
    function GetControl: TWinControl;
    procedure SetButClick(ButtDown: Boolean);
    procedure SetOnMenuClick(ButtDown: Boolean);
    procedure GetOnEvent(Sender: TObject);
    function GetShowHint: Boolean;
    procedure SetShowHint(const Value: Boolean);
   public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    property Left: Integer read GetLeft write SetLeft;
   published
    property Name: String read FGName write SetName;
    property Style: TButtstyle read GetButStyle write SetButStyle default ttButton;
    property ImageIndex: TImageIndex read GetImageIndex write SetImageIndex;
    property Enabled: Boolean read GetEnabled write SetEnabled;
    property Text: String read FText write SetText;
    property Hint: String read FHint write SetHint;
    property Width: Integer read GetWidth write SetWidth;
    property ComboDropDown: TStrings read GetDropDown write SetDropDown;
    property ComboStyle: TComboBoxStyle read GetComboStyle write SetComboStyle;
    property ComboCount: Integer read GetComboCount;
    property DropdownMenu: TTPopupMenu read GetDropdownMenu write SetDropdownMenu;
    property Visible: Boolean read GetVisible write SetVisible;
    property Control: TWinControl read GetControl;
    property ButtClick: TButtDownEvent read SetButtClick write SetButtClick;
    property MenuDown: TButtDownEvent read FOnMenuDown write FOnMenuDown;
    property MenuUp: TButtDownEvent read FOnMenuUp write FOnMenuUp;
    property ShowHint: Boolean read GetShowHint write SetShowHint;
 end;

 TButts = class(TCollection)
  private
    FOwner: TabTBar;
    function GetItem(Index: Integer): TButt;
    procedure SetItem(Index: Integer; Value: TButt);
  protected
    function GetOwner: TPersistent; override;
  public
    constructor Create(AOwner: TabTBar);
    function Add: TButt;
    function Owner: TabTBar;
    property Items[Index: Integer]: TButt read GetItem write SetItem; default;
  end;

 TabTBar = class(TVCLControl)
  private
    FButs: TButts;
    FFlat: Boolean;
    FGroup: Integer;
    FBMDown: TBitMap;
    FBM: TBitMap;
    FImages: TCustomImageList;
    FImageChangeLink: TChangeLink;
    FHotImages: TCustomImageList;
    FHotImageChangeLink: TChangeLink;
    FDisabledImages: TCustomImageList;
    FDisabledImageChangeLink: TChangeLink;

    procedure SetButs(const Value: TButts);
    procedure SetFlat(const Value: Boolean);
    procedure ImageListChange(Sender: TObject);
    procedure SetDisabledImages(const Value: TCustomImageList);
    procedure SetImages(const Value: TCustomImageList);
    procedure SetHotImages(const Value: TCustomImageList);
    procedure WMSize(var Msg: TWMSize); message WM_SIZE;
  protected
    procedure UpdateImages;
  public
    destructor Destroy; override;
    procedure Paint; override;
//    procedure InsertControl(AControl: TControl); reintroduce;
  published
    property Buttons: TButts read FButs write SetButs;
    property Flat: Boolean read FFlat write SetFlat default False;
    property DisabledImages: TCustomImageList read FDisabledImages write SetDisabledImages;
    property HotImages: TCustomImageList read FHotImages write SetHotImages;
    property Images: TCustomImageList read FImages write SetImages;
 end;

 TTBar = class(TabTBar)
  public
    constructor Create(AOwner: TComponent); override;
 end;

procedure Register;

implementation

{ TTBar }

constructor TTBar.Create(AOwner: TComponent);
begin
  inherited;
  FButs := TButts.Create(Self);
  FButs.FOwner := Self;
  DoubleBuffered := False;
  Height := 29;
  Align := alTop;
  FFlat := false;
  FGroup := -1;
  FImageChangeLink := TChangeLink.Create;
  FImageChangeLink.OnChange := ImageListChange;
  FDisabledImageChangeLink := TChangeLink.Create;
  FDisabledImageChangeLink.OnChange := ImageListChange;
  FHotImageChangeLink := TChangeLink.Create;
  FHotImageChangeLink.OnChange := ImageListChange;

  FBMDown := TBitMap.Create;
  FBMDown.Handle := LoadBitmap(0, MakeIntResource(OBM_COMBO));
  FBMDown.Transparent := True;          //transparency prepare
  FBMDown.TransparentMode := tmFixed;
  FBMDown.TransparentColor := FBMDown.Canvas.Pixels[0, FBMDown.Height - 1];
  FBMDown.Canvas.CopyMode := cmSrcCopy;

  FBM := TBitMap.Create;
  FBM.Transparent := True;          //transparency prepare
  FBM.TransparentMode := tmFixed;
  FBM.Canvas.CopyMode := cmSrcCopy;
end;

destructor TabTBar.Destroy;
begin
  FButs.Free;
  FBMDown.Free;
  FBM.Free;
  inherited;
end;

procedure TabTBar.ImageListChange(Sender: TObject);
begin
 if HandleAllocated then Invalidate;
end;

procedure TabTBar.Paint;
Var I, P: Integer;
    R: TRect;
begin
  inherited;
  R := Rect(0, 0, Width, Height);
  Canvas.Brush.Color := Color;
  Canvas.FillRect(R);
  R := ClientRect; r.Bottom := R.Bottom + 1;
  DrawEdge(Canvas.Handle, R, EDGE_ETCHED, BF_BOTTOM);
  P := 1;
  For I := 0 to FButs.Count -1 do
   If FButs.Items[I].Visible then
    begin
      FButs.Items[I].Left := P;
      P := FButs.Items[I].Width + P;
    end;
end;

procedure TabTBar.SetButs(const Value: TButts);
begin
   FButs.Assign(Value);
end;

procedure TabTBar.SetHotImages(const Value: TCustomImageList);
begin
    if FHotImages <> nil then FHotImages.UnRegisterChanges(FHotImageChangeLink);
  FHotImages := Value;
  if FHotImages <> nil then
  begin
    FHotImages.RegisterChanges(FHotImageChangeLink);
    FHotImages.FreeNotification(Self);
  end
  else if FHotImages <> nil then Perform(TB_SETHOTIMAGELIST, 0, FHotImages.Handle);
 Invalidate;
end;

procedure TabTBar.SetImages(const Value: TCustomImageList);
begin
  if FImages <> nil then FImages.UnRegisterChanges(FImageChangeLink);
  FImages := Value;
  if FImages <> nil then
  begin
    FImages.RegisterChanges(FImageChangeLink);
    FImages.FreeNotification(Self);
  end
  else if FImages <> nil then Perform(TB_SETIMAGELIST, 0, FImages.Handle);
  Invalidate;
end;

procedure TabTBar.UpdateImages;
begin
  If HandleAllocated then
   Begin
    if FDisabledImages <> nil
         then Perform(TB_SETDISABLEDIMAGELIST, 0, FDisabledImages.Handle);
    if FImages <> nil then Perform(TB_SETIMAGELIST, 0, FImages.Handle);
    if FHotImages <> nil then Perform(TB_SETHOTIMAGELIST, 0, FHotImages.Handle);
    Invalidate;
   end;
end;

procedure TabTBar.SetDisabledImages(const Value: TCustomImageList);
begin
   if FDisabledImages <> nil then FDisabledImages.UnRegisterChanges(FDisabledImageChangeLink);
  FDisabledImages := Value;
  if FDisabledImages <> nil then
  begin
    FDisabledImages.RegisterChanges(FDisabledImageChangeLink);
    FDisabledImages.FreeNotification(Self);
  end
  else
    if FDisabledImages <> nil
         then Perform(TB_SETDISABLEDIMAGELIST, 0, FDisabledImages.Handle);
  Invalidate;
end;

procedure TabTBar.SetFlat(const Value: Boolean);
begin
 if FFlat <> Value then
  begin
    FFlat := Value;
    Repaint;
  end;
end;

procedure TabTBar.WMSize(var Msg: TWMSize);
Var I: Integer;
begin
  inherited;
  For I := 0 to FButs.Count -1 do
   with FButs.Items[I] do
     If FGBut <> nil then
    begin
      If FGBut.FBButStyle in [ttButton, ttToggle, ttGroup]
        then FGBut.Width := Msg.Height - 3 else
      If FGBut.FBButStyle in [ttDropDown]
        then FGBut.Width := Msg.Height  - 3 + (((Msg.Height - 3) div 3) * 2);
      FGBut.Height := Msg.Height - 3;
    end;
end;

{ TButts }

function TButts.Add: TButt;
begin
  Result := TButt(inherited Add);
end;

constructor TButts.Create(AOwner: TabTBar);
begin
  inherited Create(TButt);
  FOwner := AOwner;
end;

function TButts.GetItem(Index: Integer): TButt;
begin
  Result := TButt(inherited GetItem(Index));
end;

function TButts.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TButts.Owner: TabTBar;
begin
  Result := FOwner;
end;

procedure TButts.SetItem(Index: Integer; Value: TButt);
begin
   inherited SetItem(Index, Value);
end;

{ TButt }

procedure TButt.Assign(Source: TPersistent);
begin
  inherited;
end;

constructor TButt.Create(Collection: TCollection);
begin
  Inherited;
  FBButs := TButts(Collection);
  SetButStyle(ttButton);
end;

destructor TButt.Destroy;
begin
  If FGBut <> nil then FGBut.Free;
  inherited;
end;

function TButt.GetDropdownMenu: TtPopupMenu;
begin
 Result := nil;
 If FGBut <> nil then Result := FGBut.FDropdownMenu;
end;

procedure TButt.SetDropdownMenu(const Value: TTPopupMenu);
begin
  If FGBut <> nil then FGBut.SetDropdownMenu(Value);
end;

function TButt.GetDropDown: TStrings;
begin
  Result := nil;
  If FCombo <> nil then Result := FCombo.Items;
end;

procedure TButt.SetDropDown(const Value: TStrings);
begin
 If FCombo <> Nil then FCombo.Items := Value;
end;

procedure TButt.SetEnabled(const Value: Boolean);
begin
  If FGBut <> nil then FGBut.Enabled := Value else
  If FCombo <> nil then FCombo.Enabled := Value else
  If FEdit <> nil then FEdit.Enabled := Value;
end;

function TButt.GetEnabled: Boolean;
begin
  If FGBut <> nil then Result := FGBut.Enabled else
  If FCombo <> nil then Result := FCombo.Enabled else
  If FEdit <> nil then Result := FEdit.Enabled else
  Result := False;
end;

procedure TButt.SetHint(const Value: String);
begin
  If FHint = Value then Exit;
  FHint := Value;
  If FGBut <> nil then FGBut.Hint := Value else
  If FCombo <> nil then FCombo.Hint := Value else
  If FEdit <> nil then FEdit.Hint := Value;
end;

procedure TButt.SetImageIndex(const Value: TImageIndex);
begin
  If FGBut <> nil then
   Begin
    FGBut.ImageIndex := Value;
    FGBut.Invalidate;
   End;
end;

procedure TButt.SetName(const Value: String);
begin
  FGBut.Name := Value;
  If FGBut <> nil then FGBut.Name := Value else
  If FCombo <> nil then FCombo.Name := Value else
  If FEdit <> nil then FEdit.Name := Value;
end;

procedure TButt.SetText(const Value: String);
begin
  FText := Value;
  If FCombo <> nil then FCombo.Text := Value else
  If FEdit <> nil then FEdit.Text := Value;
end;

function TButt.GetLeft: Integer;
begin
  Result := 0;
  If FGBut <> nil then Result := FGBut.Left else
  If FCombo <> nil then Result := FCombo.Left else
  If FEdit <> nil then Result := FEdit.Left;
end;

procedure TButt.SetLeft(const Value: Integer);
begin
  If FGBut <> nil then FGBut.Left := Value else
  If FCombo <> nil then FCombo.Left := Value else
  If FEdit <> nil then FEdit.Left := Value;
end;

function TButt.GetWidth: Integer;
begin
  Result := 0;
  If FGBut <> nil then Result := FGBut.Width else
  If FCombo <> nil then Result := FCombo.Width else
  If FEdit <> nil then Result := FEdit.Width;
end;

function TButt.GetImageIndex: TImageIndex;
begin
  If FGBut <> nil then Result := FGBut.ImageIndex
                  else Result := -1;
end;

function TButt.GetVisible: Boolean;
begin
  Result := False;
  If FGBut <> nil then Result := FGBut.Visible else
  If FCombo <> nil then Result := FCombo.Visible else
  If FEdit <> nil then Result := FEdit.Visible;
end;

procedure TButt.SetVisible(const Value: Boolean);
begin
  If FGBut <> nil then FGBut.Visible := Value else
  If FCombo <> nil then FCombo.Visible := Value else
  If FEdit <> nil then FEdit.Visible := Value;
end;

function TButt.GetComboStyle: TComboBoxStyle;
begin
  If FCombo <> nil then Result := FCombo.Style
                   else Result := csOwnerDrawVariable;
end;

procedure TButt.SetComboStyle(const Value: TComboBoxStyle);
begin
  If FCombo <> nil then FCombo.Style := Value;
end;

function TButt.GetComboCount: Integer;
begin
  If FCombo <> nil then Result := FCombo.DropDownCount
                   else Result := -1;
end;

procedure TButt.SetWidth(const Value: Integer);
begin
  If FCombo <> nil then FCombo.Width := Value else
  If FEdit <> nil then FEdit.Width := Value;
end;

function TButt.GetControl: TWinControl;
begin
  Result := nil;
  If FCombo <> nil then Result := FCombo else
  If FEdit <> nil then Result := FEdit;
end;

procedure TButt.SetOnMenuClick(ButtDown: Boolean);
begin
 If Assigned(FOnMenuDown) Then FOnMenuDown(ButtDown);
end;

function TButt.GetButStyle: TButtstyle;
begin
  Result := ttToggle;
  If FGBut <> nil then Result := FGBut.Style else
  If FCombo <> nil then Result := ttComboBox else
  If FEdit <> nil then Result := ttEditor;
end;

procedure TButt.SetButStyle(const Value: TButtstyle);
begin
 If (Value <> GetButStyle) then
  If (Value in[ttButton, ttToggle, ttDivider, ttDropDown, ttGroup, ttSeparator])
    then begin
      If FCombo <> nil then begin FCombo.Destroy; FCombo := nil; end else
      If FEdit <> nil then begin FEdit.Destroy; FEdit := nil; end;
      If (FGBut = nil) then
        Begin
          FGBut := TGBut.Create(FBButs.FOwner);
          FGBut.Parent := FBButs.FOwner;
          FGBut.FBBut := Self;
          FGBut.ButtClick := SetButClick;
          FGBut.MenuClick := SetOnMenuClick;
          FGBut.OnClick := GetOnEvent;
         end;
          FGBut.SetButStyle(Value);
          TabTBar(FBButs.FOwner).Invalidate;

         end
  else
  begin
   If (Value = ttComboBox) and (FCombo = nil) then
    begin
      If FGBut <> nil then begin FGBut.Destroy; FGBut := nil end else
      If FEdit <> nil then begin FEdit.Destroy; FEdit := nil; end;
      FCombo := TComboBox.Create(FBButs.FOwner);
      FCombo.Parent := FBButs.FOwner;
      FCombo.Height := 22;
      FCombo.Width := 75;
      FCombo.Top := 2;
      FCombo.OnChange := GetOnEvent;
      TabTBar(FBButs.FOwner).Invalidate;
    end else
   If (Value = ttEditor) and (FEdit = nil) then
    begin
      If FGBut <> nil then begin FGBut.Destroy; FGBut := nil end else
      If FCombo <> nil then begin FCombo.Destroy; FCombo := nil; end;
      FEdit := TEdit.Create(FBButs.FOwner);
      FEdit.Parent := FBButs.FOwner;
      FEdit.Height := 22;
      FEdit.Width := 55;
      FEdit.Top := 2;
      FEdit.OnChange := GetOnEvent;
      TabTBar(FBButs.FOwner).Invalidate;
    end;
  end;
end;

procedure TButt.GetOnEvent(Sender: TObject);
begin
 If Assigned(ButtClick) then ButtClick(True);
end;

procedure TButt.SetButClick(ButtDown: Boolean);
begin
 if Assigned(SetButtClick) then SetButtClick(ButtDown);
end;

function TButt.GetShowHint: Boolean;
begin
  If FGBut <> nil then Result := FGBut.ShowHint else
  If FCombo <> nil then Result := FCombo.ShowHint else
  If FEdit <> nil then Result := FEdit.ShowHint else
  Result := False;
end;

procedure TButt.SetShowHint(const Value: Boolean);
begin
  If FGBut <> nil then FGBut.ShowHint := Value else
  If FCombo <> nil then FCombo.ShowHint := Value else
  If FEdit <> nil then FEdit.ShowHint := Value;
end;

{ TabGBut }

constructor TGBut.Create(AOwner: TComponent);
begin
  inherited;
  FBButStyle := ttEditor;
  FBorderWidth := 3;
  FImageIndex := -1;
  Width := 22;
  Height := 22;
  FEnabled := True;
  Left := 1;
  Top := 0;
end;

procedure TabGBut.NCPaint(DC: HDC);
const
  OuterStyles: array[TBEdgeStyle] of Integer = (0, BDR_RAISEDOUTER, BDR_SUNKENOUTER);
var
  RC, RW: TRect;
begin
  Windows.GetClientRect(Canvas.Handle, RC);
  GetWindowRect(Canvas.Handle, RW);
  MapWindowPoints(0, Canvas.Handle, RW, 2);
  OffsetRect(RC, -RW.Left, -RW.Top);
  ExcludeClipRect(DC, RC.Left, RC.Top, RC.Right, RC.Bottom);
  { Draw borders in non-client area }
  OffsetRect(RW, -RW.Left, -RW.Top);
  DrawEdge(DC, RW, OuterStyles[FEdgeOuter], BF_ADJUST);
  { Erase parts not drawn }
  IntersectClipRect(DC, RW.Left, RW.Top, RW.Right, RW.Bottom);
  Windows.FillRect(DC, RW, Canvas.Brush.Handle);
end;

procedure TabGBut.SetEdgeOuter(const Value: TBEdgeStyle);
begin
  FEdgeOuter := Value;
end;

procedure TabGBut.WMNCPaint(var Message: TMessage);
var
  DC: HDC;
begin
  { Get window DC that is clipped to the non-client area }
  DC := GetWindowDC(Canvas.Handle);
  try
    NCPaint(DC);
  finally
    ReleaseDC(Canvas.Handle, DC);
  end;
end;

procedure TabGBut.WMNCCalcSize(var Message: TWMNCCalcSize);
var
  EdgeSize: Integer;
begin
  with Message.CalcSize_Params^ do
  begin
    InflateRect(rgrc[0], -FBorderWidth, -FBorderWidth);
    if EdgeOuter <> esNone then EdgeSize := 0
       else EdgeSize := 1;
    with rgrc[0] do
    begin
       Inc(Left, EdgeSize);
       Inc(Top, EdgeSize);
       Dec(Right, EdgeSize);
       Dec(Bottom, EdgeSize);
    end;
  end;
  inherited;
end;

procedure TabGBut.CMBorderChanged(var Message: TMessage);
begin
  Repaint;
  inherited;
end;

procedure TabGBut.CMCtl3DChanged(var Message: TMessage);
begin
  Repaint;
  inherited;
end;

procedure TabGBut.Paint;
const
  XorColor = $00FFD8CE;
  DropDownWidth = 14;
var
  R: TRect;
  Flags: Integer;
  ImageBM: TBitMap;

 procedure PaintIcon(Images: TCustomImageList);
 begin
  If Images.Count > FImageIndex then
   If (Height = 22) then
        Images.Draw(Canvas, 3, 3, FImageIndex, dsTransparent, itImage)
   else
   begin
     ImageBM.Width := Images.Width; ImageBM.Height := Images.Height;
     ImageBM.Canvas.Brush.Color := clBtnFace;
     ImageBM.Canvas.FillRect(clientRect);
     ImageBM.TransparentColor := ImageBM.Canvas.Pixels[0, ImageBM.Height - 1];
     Images.Draw(ImageBM.Canvas, 0, 0,  FImageIndex, dsTransparent,  itImage);
     StretchBlt(Canvas.Handle, 3, 3, R.Right - 6, R.Bottom - 6,
        ImageBM.Canvas.Handle, 0, 0, Images.Width, Images.Height, SRCCOPY);
   end;
 end;

 procedure PaintIcons;
 begin
  If Enabled then
   begin
    If FDown and (TabTBar(Parent).FHotImages <> nil)
     then PaintIcon(TabTBar(Parent).FHotImages)
       else if (TabTBar(Parent).FImages <> nil)
             then PaintIcon(TabTBar(Parent).FImages);
    If not FDown and (TabTBar(Parent).FImages <> nil)
     then PaintIcon(TabTBar(Parent).FImages);
   end else If TabTBar(Parent).FDisabledImages <> nil
             then PaintIcon(TabTBar(Parent).FDisabledImages)
             else If (TabTBar(Parent).FImages <> nil)
                   then PaintIcon(TabTBar(Parent).FImages);
 end;

begin
 Inherited;
 If FBBut = nil then Exit;
 ImageBM := TabTBar(Parent).FBM;
// BImages := TabTBar(Parent).FImages;

 Canvas.Brush.Color := clBtnFace;
 Canvas.FillRect(R);
 Canvas.Brush.Color := Color;
 R := Rect(0, 0, Width, Height);

  if Style in[ttButton, ttToggle, ttGroup] then
   with Canvas do
    begin
     //TTBar(Parent).Images.Draw(Canvas, 0, 0, ImageIndex, );
     If not TabTBar(Parent).Flat then
      begin
     R := Rect(1, 1, Width, Height);
     Flags := DFCS_BUTTONPUSH or DFCS_ADJUSTRECT;
     If Down then Flags := Flags or DFCS_PUSHED;
     DrawFrameControl(Canvas.Handle, R, DFC_BUTTON, Flags);
     PaintIcons;
     Exit;
      end;
    end;

  if (Style = ttDropDown) then
    with Canvas do
    begin
     If not TabTBar(Parent).Flat then
      begin
     R := Rect(1, 1, Height, Height);
     Flags := DFCS_BUTTONPUSH or DFCS_ADJUSTRECT;
     If FDown then Flags := Flags or DFCS_PUSHED;
     DrawFrameControl(Canvas.Handle, R, DFC_BUTTON, Flags);

     R := Rect(Height, 1, Width, Height);
     Flags := DFCS_BUTTONPUSH or DFCS_ADJUSTRECT or DFCS_MENUARROW;
     If FDownButDown then Flags := Flags or DFCS_PUSHED;
     DrawFrameControl(Canvas.Handle, R, DFC_BUTTON, Flags);
     Canvas.CopyMode := cmSrcCopy;
     R := Rect(Height, 4, Width -1, height - 4);
     Canvas.StretchDraw(R, TabTBar(Parent).FBMDown);
      End;
     R := Rect(0, 0, Height, Height);
     PaintIcons;
    end;

  if Style = ttDivider then
    with Canvas do
    begin
      R := Rect(Width div 2, 2, Width, Height - 2);
      DrawEdge(Handle, R, EDGE_ETCHED, BF_LEFT)
    end;

  if (Style in [ttSeparator]) and (csDesigning in ComponentState) then
      with Canvas do
      begin
        Pen.Style := psDot;
        Pen.Mode := pmXor;
        Pen.Color := XorColor;
        Brush.Style := bsClear;
        Rectangle(0, 0, ClientWidth, ClientHeight);
      end
    else if (TabTBar(Parent).Flat) and not Down then
    begin
      R := Rect(0, 0, Width, Height);
      PaintIcons;
    end else
    Begin
      R := ClientRect; R.Top := R.Top + 1; R.Bottom := R.Bottom +1;
      DrawEdge(Canvas.Handle, R, EDGE_ETCHED, BF_RECT);
    end;
end;

procedure TabGBut.SetButStyle(const Value: TButtstyle);
begin
 if FBButStyle <> Value then
  begin
    FBButStyle := Value;
    If FBButStyle = ttButton then Width := Height + 1;
    If FBButStyle = ttToggle then Width := Height + 1;
    If FBButStyle = ttDropDown then Width := Height + ((Height div 3) * 2);
    If FBButStyle = ttGroup then Width := Height + 1;
    If FBButStyle = ttSeparator then Width := 8;
    If FBButStyle = ttDivider then Width := 8;
    Repaint;
  end;
end;

function TabGBut.IsCheckedStored: Boolean;
begin
result :=  false;
end;

procedure TabGBut.SetDown(const Value: Boolean);
begin
  FDown := Value;
end;

procedure TabGBut.SetImageIndex(const Value: TImageIndex);
begin
  FImageIndex := Value;
end;

procedure TabGBut.MouseDown(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
Var I: Integer;

begin
  inherited;
  If FBButStyle = ttButton then FDown := True;
  If FBButStyle = ttToggle then
    begin
      FCheck := not FCheck;
      FDown := FCheck;
    end;
   If FBButStyle = ttDropDown then
    Begin
     If X > Height {Down But} then
      begin
       FDownButDown := True;
       If Assigned(FOnMenuDown) then FOnMenuDown(FDownButDown);
       Invalidate;
     end else
      begin
       FDown := True;
       If Assigned(SetButtClick) then SetButtClick(FDown);
       Invalidate;
      end;
    end;
   If FBButStyle = ttGroup then
   begin
     I := FBBut.Index;
     TabTBar(Parent).FGroup := I;
     FDown := True;
   End;
 TabTBar(Parent).Invalidate;
end;

procedure TabGBut.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
Var I, G: Integer;
  aControl: TControl;
    P: TPoint;
begin
  inherited;
  If FBButStyle = ttButton then FDown := False;
  If FBButStyle = ttDropDown then
    begin
     FDown := False;
     FDownButDown := False;
     P := ClientToScreen(Point(0, Height));
       If not (csDesigning in ComponentState) and ((FDropdownMenu <> nil) and
         FDropdownMenu.AutoPopup) then FDropdownMenu.PopupMsg(p.X, P.Y, Self);
    end;
   If FBButStyle = ttGroup then
    begin
     G := TabTBar(Parent).FGroup;
        For I := 0 to TabTBar(Parent).ControlCount -1 do
     begin
      aControl := TabTBar(Parent).Controls[I];
      If (TabGBut(aControl).FBButStyle <> ttToggle) then
      If (TabGBut(aControl).FBBut.Index <> G) then
          TabGBut(aControl).FDown := False;
     end;
     If G = FBBut.Index then else FDown := True;
    end;
 Case FBButStyle of
   ttButton, ttGroup, ttToggle, ttDropDown:
            If Assigned(SetButtClick) then SetButtClick(FDownButDown);
 end;
  If (FBButStyle = ttDropDown) and Assigned(FOnMenuDown)
      then FOnMenuDown(FDownButDown);

 TabTBar(Parent).Invalidate;
end;

procedure TabGBut.SetDropdownMenu(const Value: TTPopupMenu);
begin
 If Value <> FDropdownMenu then
  begin
    FDropdownMenu := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

function TabGBut.GetVisible: Boolean;
begin
 Result := inherited Visible;
end;

procedure TabGBut.SetVisible(const Value: Boolean);
begin
   inherited Visible := Value;
   TabTBar(Parent).Invalidate;
end;

procedure TabGBut.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
inherited Notification(AComponent, Operation);
  if Operation = opRemove then
    if AComponent = FDropdownMenu then
      FDropdownMenu := nil
end;

procedure TabGBut.SetEnabled(Value: Boolean);
begin
  FEnabled := Value;
  TabTBar(Parent).Invalidate;
end;

procedure TabGBut.CMMENUCHANGED(var Message);
begin
  Invalidate;
end;

procedure TabGBut.CM_LOSTFOCUS(var Message);
begin
  Invalidate;
end;

{ TTMenu }

procedure TTPopupMenu.MenuChanged(Sender: TObject; Source: TMenuItem;
  Rebuild: Boolean);
begin
  inherited;
  if ComponentState * [csLoading, csDestroying] = [] then
    if (FReturnMsg <> nil) and not Rebuild then
      FReturnMsg.Perform(CM_MENUCHANGED, 0, 0);
end;

procedure TTPopupMenu.PopUpMsg(X, Y: Integer; ReturnMsg: TControl);
begin
 FReturnMsg := ReturnMsg;
 inherited PopUp(X, Y);
end;

procedure Register;
Begin
  RegisterComponents('T Project', [TTBar, TTPopupMenu]);
End;

end.
