{*******************************************************}
{                                                       }
{       Borland Delphi Visual Component Library         }
{                                                       }
{  Copyright (c) 1995-2002 Borland Software Corporation }
{                                                       }
{*******************************************************}

unit TMenus;

{$S-,W-,R-,T-,H+,X+}
{$C PRELOAD}

interface

{$IFDEF LINUX}
uses WinUtils, Windows, SysUtils, Classes, Contnrs, Messages, Graphics, ImgList,
     ActnList, Controls, Menus, MiniReg;
{$ENDIF}
{$IFDEF MSWINDOWS}
uses Windows, SysUtils, Classes, Contnrs, Messages, Graphics, ImgList, ActnList,
     Controls, Menus, MiniReg;
{$ENDIF}

type
 TTIndexStringEvent = procedure(Sender: TObject; Caption: String) of object;

 TTPopupMenu = class(TPopupMenu)
  private
    FPopupControl: TControl;
    FOnClick: TTIndexStringEvent;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Popup(X, Y: Integer; aPopupControl: TControl = nil); reintroduce; virtual;
    procedure Click(Sender: TObject); virtual;
    property PopupControl: TControl read FPopupControl write FPopupControl;
  published
    property OnClick: TTIndexStringEvent read FOnClick write FOnClick;
 end;

resourcestring
  SInvalidIniSectionName = 'Invalid IniFile section name.';
  SSubMenuCaption = 'Recent';
  SRecentCount = '\FileCount';

const
  SValidNumbers: AnsiString = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ';

type
 TpmStorageMode = (rfRegistry, rfStrings);

  TTResFilesMenuStorage = class(TPersistent)
  private
    FRegistryPath: AnsiString;
    FRegistryKey: TRegistryKey;
    FFileName: AnsiString;
    FMode: TpmStorageMode;
    FIniSection: AnsiString;
  protected
    function GetRegistryKey: HKEY;
  public
    constructor Create; virtual;
  published
    property FileName: AnsiString read FFileName write FFileName;
    property Mode: TpmStorageMode read FMode write FMode default rfRegistry;
    property RegistryKey: TRegistryKey read FRegistryKey write FRegistryKey default rkLOCAL_MACHINE;
    property RegistryPath: AnsiString read FRegistryPath write FRegistryPath;
  end;

  TTRmMaxCount = 3..27{letters}+10{numbers};
  TTrmOption = (rmoDividerBefore, rmoDividerAfter, rmoShowNumbers);
  TTrmOptions = set of TTrmOption;
  TTrmPoistion = (rmpBeforeItem, rmpAfterItem);

  TTResFilesMenu = class(TTPopupMenu)
  private
    FList: TStrings;
    FStorage: TTResFilesMenuStorage;
    FSourceMenuItem: TMenuItem;
    FMaxCount: TTRmMaxCount;
    FSubMenuCaption: TCaption;
    FNoIndexFiles: Boolean;
    FFileListCount: Byte;
    procedure SetSourceMenuItem(const Value: TMenuItem);
  protected
    FDividers: array[0..1] of TMenuItem;
    FSubMenuItem: TMenuItem;
    procedure CheckItems;
    procedure ClearItems;
    procedure MoveToTop(const Index: Integer);
    procedure OnMenuItemClick(Sender: TObject);
    procedure ReadFromRegistry;
    procedure WriteToRegistry(List: TStrings);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function AddFileName(const S: String; Visible: Boolean): Boolean;
    function DeleteFileName(const S: String): Integer;
    function IndexOf(const S: AnsiString): Integer;
    function SetFileNameVisible(S: String; Visible: Boolean): Boolean;
    procedure Clear;
    procedure ReadStorage;
    procedure WriteStorage;
    procedure GetList(List: TStrings);
    procedure Popup(X, Y: Integer; aPopupControl: TControl = nil); OverRide;
    property Lines: TStrings read FList;
  published
    property MaxItemCount: TTRmMaxCount read FMaxCount write FMaxCount default 10;
    property SourceMenuItem: TMenuItem read FSourceMenuItem write SetSourceMenuItem;
    property Storage: TTResFilesMenuStorage read FStorage write FStorage;
    property NoIndexFiles: Boolean read FNoIndexFiles write FNoIndexFiles;
    property FileListCount: Byte read FFileListCount write FFileListCount;
   end;

procedure Register;

implementation

Function Niled(Const P): Boolean;
Begin
 Result := (Pointer(P) = nil);
end;

Function NotNiled(Const P): Boolean;
Begin
 Result := not (Pointer(P) = nil);
end;

{ TPopupMenu }

constructor TTPopupMenu.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  OwnerDraw := True;
end;

procedure TTPopupMenu.Popup(X, Y: Integer; aPopupControl: TControl = nil);
Var I: Integer;
Begin
  PopupControl := aPopupControl;
  Inherited Popup(X, Y);
  For I := 0 to Items.Count -1
   do Items[I].OnClick := Click;
end;

procedure TTPopupMenu.Click(Sender: TObject);
begin
 inherited;
 If Assigned(PopupControl)
  and (ComponentState * [csLoading, csDestroying] = [])
  then TControl(PopupControl).
          Perform(CM_MENUCHANGED, Integer(Self), Integer(Sender));
  If NotNiled(FOnClick) then FOnClick(Sender, TMenuItem(Sender).Caption);
end;

{ TTResFilesMenuStorage }

constructor TTResFilesMenuStorage.Create;
begin
  FFileName := GetCurrentDir + '\ResentFiles.Lst';
  FIniSection := 'Recent';
  FMode := rfRegistry;
  FRegistryKey := rkLOCAL_MACHINE;
  FRegistryPath := 'Software\Product\Recent Files';
end;

function TTResFilesMenuStorage.GetRegistryKey: HKEY;
begin
  case FRegistryKey of
    rkCLASSES_ROOT:   Result := HKEY_CLASSES_ROOT;
    rkCURRENT_USER:   Result := HKEY_CURRENT_USER;
    rkLOCAL_MACHINE:  Result := HKEY_LOCAL_MACHINE;
    rkUSERS:          Result := HKEY_USERS;
    rkCURRENT_CONFIG: Result := HKEY_CURRENT_CONFIG;
   else Result := 0;
  end;
end;

{ TTResFilesMenu }

function TTResFilesMenu.AddFileName(const S: AnsiString;
                                    Visible: Boolean): Boolean;
Var I: Integer;
begin
  Result := False;
  I := IndexOf(S);
  If I > -1 then
  begin
    If I > 0 then
    begin
      FList.Move(I, 0);
      WriteStorage;
      Result := False;
    end;
  end
  else
  begin
    Result := True;
    If Visible
      then FList.Insert(0, S)
      else FList.Insert(0, '^' + S);
    WriteStorage;
  end;
  While FList.Count > FileListCount do FList.Delete(FList.Count - 1);
end;

procedure TTResFilesMenu.CheckItems;
var
  I: Integer;
begin
  // Check empty items
  for I := FList.Count - 1 downto 0 do
    if FList[ I ] = '' then
      FList.Delete( I );
  // Check range for MaxCount
  for I := FList.Count - 1 downto FMaxCount do
      FList.Delete( I );
end;

procedure TTResFilesMenu.Clear;
begin
  ClearItems;
end;

procedure TTResFilesMenu.ClearItems;
begin
  FList.Clear;
end;

constructor TTResFilesMenu.Create(AOwner: TComponent);
begin
  inherited Create( AOwner );
  FStorage := TTResFilesMenuStorage.Create;
  FList := TStringList.Create;
  FMaxCount := 10;
  FSubMenuCaption := LoadResString(@SSubMenuCaption);
  FFileListCount := 10;
end;

destructor TTResFilesMenu.Destroy;
begin
 Try
  WriteStorage;
 Finally;
  ClearItems;
  FStorage.Free;
  FList.Free;
 end;
  inherited Destroy;
end;

function TTResFilesMenu.IndexOf(Const S: AnsiString): Integer;
var Str: String;
begin
  for Result := 0 to FList.Count - 1 do
   Begin
    Str := FList[Result];
    If (Str[1] = '^') Then Str := Copy(Str, 2, 255);
    If AnsiCompareText(Str, S) = 0 then Exit;
   End;
  Result := -1;
end;

procedure TTResFilesMenu.MoveToTop(const Index: Integer);
begin
  If (Index < 1) or (Index > FList.Count - 1) then
    else FList.Move(Index, 0);
end;

procedure TTResFilesMenu.OnMenuItemClick(Sender: TObject);
var
  Index: Integer;
begin
  Index := FList.IndexOfObject(Sender);
  if Index = -1 then Exit;
  if Index > 0 then
  begin
    FList.Move(Index, 0);
    WriteStorage;
  end;
end;

procedure TTResFilesMenu.SetSourceMenuItem(const Value: TMenuItem);
begin
  FSourceMenuItem := Value;
end;

procedure TTResFilesMenu.ReadStorage;
begin
  ClearItems;
  case FStorage.Mode of
    rfRegistry: ReadFromRegistry;
    rfStrings: If FileExists(Storage.FileName)
      then FList.LoadFromFile(Storage.FFileName)
      else Exit;
  end;
  CheckItems;
end;

procedure TTResFilesMenu.ReadFromRegistry;
var
  I: Integer;
  DW, Count: DWord;
  KeyName: String;
  RootKey: HKEY;
begin
   RootKey := Storage.GetRegistryKey;
   KeyName := Storage.FRegistryPath;
    If RegKeyExists(RootKey, KeyName) Then
    begin
      RegGetDWORD(RootKey, KeyName + SRecentCount, Count);
      For I := 0 to Count - 1 do
      begin
        KeyName := Storage.FRegistryPath + '\No' + IntToStr(I);
        If RegGetValueType(RootKey, KeyName, DW) And (DW = REG_SZ)
         and RegGetString(RootKey, KeyName, KeyName)
          then FList.Add(KeyName);
      end;
   end;
end;

procedure TTResFilesMenu.WriteStorage;
Var SList: TStrings;
    I: Integer;
    s: String;
begin
    SList := TStringlist.Create;
    For I := 0 to FList.Count -1
     do begin
         S := FList.Strings[I];
         If S[2] = ' ' then S := Copy(S, 3, 255);
         SList.Add(S);
        end;

  case FStorage.Mode of
    rfRegistry: WriteToRegistry(SList);
    rfStrings: SList.SaveToFile(Storage.FileName);
  end;
  SList.Free;
end;

procedure TTResFilesMenu.WriteToRegistry(List: TStrings);
var
  I: Integer;
  KeyName: String;
  RootKey: HKEY;
begin
   RootKey := Storage.GetRegistryKey;
   KeyName := Storage.FRegistryPath;
   RegSetDWORD(RootKey, KeyName + SRecentCount, DWord(FList.Count));
   For I := 0 to List.Count - 1 do
    begin
     KeyName := Storage.FRegistryPath + '\No' + IntToStr(I);
     RegDelValue(RootKey, KeyName);
     RegSetString(RootKey, KeyName, List[I]);
    end;
end;

procedure TTResFilesMenu.GetList(List: TStrings);
begin
   List.Assign(FList);
end;

procedure TTResFilesMenu.Popup(X, Y: Integer; aPopupControl: TControl);
Var I, C: Integer;
    Str: String;
begin
 OwnerDraw := True;
  C := 0;
  If FNoIndexFiles then
   Begin
    For I := 0 to FList.Count - 1
        do begin
            Str := FList.Strings[I];
            If Str[1] <> '^' then
              begin
               If Items.Count <= C then Items.Add(TMenuItem.Create(Self));
               Items[C].Caption := FList.Strings[I];
               Inc(C);
              end;
              while Items.Count > C do Items.Delete(Items.Count -1);
           end;
   end else
   Begin
   For I := 0 to FList.Count - 1
        do begin
            Str := FList.Strings[I];
            If Str[1] <> '^' then
              begin
               If Items.Count <= C then Items.Add(TMenuItem.Create(Self));
               Items[C].Caption := IntToStr(I) + ' ' + FList.Strings[I];
               Inc(C);
              end;
              while Items.Count > C do Items.Delete(Items.Count -1);
            end;
   end;
 inherited;
end;

function TTResFilesMenu.SetFileNameVisible(S: String;
  Visible: Boolean): Boolean;
Var I: Integer;
begin
 Result := False;
 I := IndexOf(S);
 If I = -1 then Exit;
 If (S[1] = '^') and Visible then
   S := Copy(S, 2, 255);
 If (S[1] = '^') and not Visible then
  S := '^' + S;
 FList.Strings[I] := S;
 Result := True;
end;

function TTResFilesMenu.DeleteFileName(const S: String): Integer;
Var I: Integer;
begin
 Result := - 1;
 I := FList.IndexOf(S);
 If I = -1 then Exit;
 Result := I;
 FList.Delete(I);
end;

procedure Register;
Begin
  RegisterComponents('T Project', [TTPopupMenu, TTResFilesMenu]);
End;

end.

