unit TUpDownEdit;

interface

uses Messages, Windows, SysUtils, Classes, Graphics, Menus, Controls, Forms,
   TVCLControl, StdCtrls, ExtCtrls, Registry;

type
  TBEdgeBorder = (ebLeft, ebTop, ebRight, ebBottom);
  TBEdgeBorders = set of TBEdgeBorder;
  TBEdgeStyle = (esNone, esRaised, esLowered);

 TTSEdit = class(TEdit)
  private
   protected
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
 end;

 TabUpDwnBut = Class(TGraphicControl)
  private
    FBorderWidth: Integer;
    FDownBut: Boolean;
    FUpBut: Boolean;
    FEdgeOuter: TBEdgeStyle;
    FBMDown: TBitMap;
    procedure SetEdgeOuter(const Value: TBEdgeStyle);
   protected
    procedure NCPaint(DC: HDC); virtual;
    procedure WMNCCalcSize(var Message: TWMNCCalcSize); message WM_NCCALCSIZE;
    procedure WMNCPaint(var Message: TMessage); message WM_NCPAINT;
    procedure CMBorderChanged(var Message: TMessage); message CM_BORDERCHANGED;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
   public
    property EdgeOuter: TBEdgeStyle read FEdgeOuter write SetEdgeOuter default esRaised;
   published
    property Canvas;
    property Name;
end;

 TUpDwnBut = Class(TabUpDwnBut)
   public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
 end;

 TTabUpDownEdit = class(TTVCLControl)
  private
   FBut: TUpDwnBut;
   FEdit: TTSEdit;
   FNo: Integer;
   FMin: Integer;
   FMax: Integer;
   FMaxSpeed: Word;
   FOnButtClick: TNotifyEvent;
   procedure SetNo(const Value: Integer);
  protected
   procedure Paint; override;
   procedure TimerEvent(ID: Integer); override;
  public
    destructor Destroy; override;
   published
    property Max: Integer read FMax write FMax;
    property Min: Integer read FMin write FMin;
    property MaxSpeed: Word read FMaxSpeed write FMaxSpeed;
    property Value: Integer read FNo write SetNo;
    property ButtClick: TNotifyEvent read FOnButtClick write FOnButtClick;
 end;

 TTUpDownEdit = class(TTabUpDownEdit)
  public
    constructor Create(AOwner: TComponent); override;
 end;

 procedure Register;

implementation



{ TTabUpDownEdit }

Const
  DownBut = 5;
  UpBut = 6;
  Speed = 7;
  MCursor = 8;

Var RSpeed: Word;
    AutoRepeatDelay: Integer;
    AutoRepeatRate: Integer;

constructor TTUpDownEdit.Create(AOwner: TComponent);
begin
  inherited;
  Width := 100;
  height := 21;
  FMaxSpeed := 10;
  FBut := TUpDwnBut.Create(Self);
  FBut.Parent := Self;
  FEdit := TTSEdit.Create(Self);
  FEdit.Parent := Self;
  FEdit.Text := '0';
  FMin := 0;
  FMax := 100;
end;

destructor TTabUpDownEdit.Destroy;
begin
  FEdit.free;
  FBut.free;
  inherited;
end;

procedure TTabUpDownEdit.Paint;
begin
  inherited;
  FEdit.Left := 0;
  FEdit.Top := 0;
  FEdit.BorderStyle := bsNone;
  FEdit.Width := Width - Height - 2;
  FEdit.Text := IntToStr(FNo);
  FEdit.SetSelStart(40);
  FBut.top := -1;
  FBut.Left := Width - Height - 1;
  FBut.Width := Height - 1;
  FBut.Height := Height - 3;
end;

procedure TTabUpDownEdit.SetNo(const Value: Integer);
begin
  FNo := Value;
  FEdit.Text := IntToStr(FNo);
end;

procedure TTabUpDownEdit.TimerEvent(ID: Integer);
begin
  inherited;
  If ID = Speed then
   begin
     If RSpeed = AutoRepeatDelay then RSpeed := AutoRepeatRate div 3
      else If RSpeed > FMaxSpeed then Dec(RSpeed, RSpeed div 7);
     SetTimer(Handle, Speed, 500, nil);
     If FBut.FUpBut then SetTimer(Handle, UpBut, RSpeed, nil) else
     If FBut.FDownBut then SetTimer(Handle, DownBut, RSpeed, nil);
     exit;
   end else
  If ID = DownBut then
   begin
     If (FNo > FMin) or ((FMin = 0) and (FMax = 0))
      then Dec(FNo);
   end else
  If ID = UpBut then
   begin
     IF (FNo < FMax) or ((FMin = 0) and (FMax = 0))
      then Inc(FNo);
   end;
   If ID = MCursor then FEdit.SetSelStart(Length(Text));
  If Assigned(FOnButtClick) then FOnButtClick(Self);
  FEdit.Text := IntToStr(FNo);
  FEdit.Invalidate;
End;

{ TabUpDwnBut }

constructor TUpDwnBut.Create(AOwner: TComponent);
begin
  inherited;
  FBorderWidth := 3;
  FBMDown := TBitMap.Create;
  FBMDown.Handle := LoadBitmap(0, MakeIntResource(OBM_COMBO));
  FBMDown.Transparent := True;          //transparency prepare
  FBMDown.TransparentMode := tmFixed;
  FBMDown.TransparentColor := FBMDown.Canvas.Pixels[0, FBMDown.Height - 1];
  FBMDown.Canvas.CopyMode := cmSrcCopy;
end;

destructor TUpDwnBut.Destroy;
begin
  FBMDown.Free;
  inherited;
end;

procedure TabUpDwnBut.Paint;
var
  R: TRect;
  Flags, Edge: Integer;
begin
 Inherited;
    with Canvas do
    begin
     R := Rect(0, 0, Width, Height);
     Canvas.Brush.Color := clBtnFace;
     Canvas.FillRect(R);
     R := Rect(0, 0, Width -1, Height -1);

     If FUpBut then Edge := EDGE_SUNKEN else Edge := EDGE_RAISED;
     Flags := BF_ADJUST	or BF_TOPLEFT;
     DrawEdge(Canvas.Handle, R, Edge, Flags);

     If FDownBut then Edge := EDGE_SUNKEN else Edge := EDGE_RAISED;
     Flags := BF_ADJUST	or BF_BOTTOMRIGHT;
     DrawEdge(Canvas.Handle, R, Edge, Flags);
     If FUpBut then
      begin
        Pen.Color := clBtnHighlight;
        MoveTo(R.Bottom, R.Top + 2);
        LineTo(R.Left, R.Bottom + 2);
      end;
     If FDownBut then
      begin
        Pen.Color := clBtnHighlight;
        MoveTo(R.Bottom - 2, R.Top);
        LineTo(R.Left - 2, R.Bottom);
      end;

     Pen.Color := cl3DDkShadow;
     MoveTo(R.Left, R.Bottom);
     LineTo(R.Bottom, R.Top);

     StretchDraw(Rect(0, 13, 14, -2), FBMDown);
     StretchDraw(Rect(5, 5, 21, 20), FBMDown);
    end;
end;

procedure TabUpDwnBut.MouseDown(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  inherited;
  If X - (Height -Y) > 0 then FDownBut := True else FUpBut := True;
  RSpeed := AutoRepeatDelay;
  If FDownBut then
   With TTabUpDownEdit(Parent) do
   begin
    If (FNo > FMin) or ((FMin = 0) and (FMax = 0)) then Dec(FNo);
    SetTimer(Parent.Handle, DownBut, RSpeed, nil);
    If Assigned(FOnButtClick) then FOnButtClick(Parent);
   end else
  If FUpBut then
   With TTabUpDownEdit(Parent) do
   begin
    IF (FNo < FMax) or ((FMin = 0) and (FMax = 0)) then Inc(FNo);
    SetTimer(Parent.Handle, UpBut, RSpeed, nil);
    If Assigned(FOnButtClick) then FOnButtClick(Parent);
   end;
  If FUpBut or FDownBut = true then
    SetTimer(Parent.Handle, Speed, RSpeed, nil);
  TTabUpDownEdit(Parent).Invalidate;
end;

procedure TabUpDwnBut.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  inherited;
  FUpBut := False;
  FDownBut := False;
  KillTimer(Parent.Handle, UpBut);
  KillTimer(Parent.Handle, DownBut);
  KillTimer(Parent.Handle, Speed);
  TTabUpDownEdit(Parent).FEdit.SetFocus;
  TTabUpDownEdit(Parent).FEdit.SetSelStart(Length(Text));
  Invalidate;
end;

procedure TabUpDwnBut.NCPaint(DC: HDC);
const
  OuterStyles: array[TBEdgeStyle] of Integer = (0, BDR_RAISEDOUTER, BDR_SUNKENOUTER);
var
  RC, RW: TRect;
begin
  Windows.GetClientRect(Canvas.Handle, RC);
  GetWindowRect(Canvas.Handle, RW);
  MapWindowPoints(0, Canvas.Handle, RW, 2);
  OffsetRect(RC, -RW.Left, -RW.Top);
  ExcludeClipRect(DC, RC.Left, RC.Top, RC.Right, RC.Bottom);
  { Draw borders in non-client area }
  OffsetRect(RW, -RW.Left, -RW.Top);
  DrawEdge(DC, RW, OuterStyles[FEdgeOuter], BF_ADJUST);
  { Erase parts not drawn }
  IntersectClipRect(DC, RW.Left, RW.Top, RW.Right, RW.Bottom);
  Windows.FillRect(DC, RW, Canvas.Brush.Handle);
end;

procedure TabUpDwnBut.SetEdgeOuter(const Value: TBEdgeStyle);
begin
  FEdgeOuter := Value;
end;

procedure TabUpDwnBut.WMNCPaint(var Message: TMessage);
var
  DC: HDC;
begin
  { Get window DC that is clipped to the non-client area }
  DC := GetWindowDC(Canvas.Handle);
  try
    NCPaint(DC);
  finally
    ReleaseDC(Canvas.Handle, DC);
  end;
end;

procedure TabUpDwnBut.WMNCCalcSize(var Message: TWMNCCalcSize);
var
  EdgeSize: Integer;
begin
  with Message.CalcSize_Params^ do
  begin
    InflateRect(rgrc[0], -FBorderWidth, -FBorderWidth);
    if EdgeOuter <> esNone then EdgeSize := 0
       else EdgeSize := 1;
    with rgrc[0] do
    begin
       Inc(Left, EdgeSize);
       Inc(Top, EdgeSize);
       Dec(Right, EdgeSize);
       Dec(Bottom, EdgeSize);
    end;
  end;
  inherited;
end;

procedure TabUpDwnBut.CMBorderChanged(var Message: TMessage);
begin
  Repaint;
  inherited;
end;

procedure TabUpDwnBut.CMCtl3DChanged(var Message: TMessage);
begin
  Repaint;
  inherited;
end;

{ TTSEdit }

procedure TTSEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited;
  case Key of
    VK_UP:
       with TTabUpDownEdit(Parent) do
        IF (FNo < FMax) or ((FMin = 0) and (FMax = 0)) then
      begin
       Inc(FNo);
       If Assigned(FOnButtClick) then FOnButtClick(Parent);
       Invalidate;
      end;
    VK_DOWN:
       with TTabUpDownEdit(Parent) do
        If (FNo > FMin) or ((FMin = 0) and (FMax = 0)) then
      begin
       Dec(FNo);
       If Assigned(FOnButtClick) then FOnButtClick(Parent);
       Invalidate;
      end;
      end;
end;

procedure TTSEdit.KeyUp(var Key: Word; Shift: TShiftState);
var S: string;
    I: Integer;
begin
  inherited;
    S := Text;
    I := 0;
    repeat
       Inc(I);
       While (Length(S) >= I) and not (S[I] in['0'..'9'])
         Do S := Copy(S, 0, I - 1) + Copy(S, I + 1, 255);
    until I >= Length(S);
    If Key in[48..57] then
      with TTabUpDownEdit(Parent) do
        begin
         FNo := StrToInt(S);
         If Assigned(FOnButtClick) then FOnButtClick(Parent);
         Invalidate;
        end;
end;

procedure TTSEdit.KeyPress(var Key: Char);
var B: Boolean;
    S: String;
begin
  B := Key = Char(VK_BACK);
  If not (Key In['0'..'9']) then Key := #0
   else if Sametext(Text, '0') then Text := '';
  if Key <> #0 then inherited;
  S := Text;
  If B then
   with TTabUpDownEdit(Parent) do
     begin
      If (Length(S) > 1) then
       begin
        S := Copy(S, 1, Length(S) - 1);
        FNo := StrToInt(S);
       end else FNo := 0;
      If Assigned(FOnButtClick) then FOnButtClick(Parent);
      Invalidate;
     end;
end;

procedure Register;
Begin
  RegisterComponents('T Project', [TTUpDownEdit]);
End;

const KeyDelay ='HKEY_CURRENT_USER\Control Panel\Accessibility\Keyboard Response';

initialization
begin
  With TRegistry.Create do
   begin
     RootKey := HKEY_LOCAL_MACHINE;
    If KeyExists(KeyDelay) then
     begin
      CreateKey(KeyDelay);
      AutoRepeatDelay := ReadInteger('AutoRepeatDelay') div 3;
      AutoRepeatRate := ReadInteger('AutoRepeatRate');
     end else
     begin
      AutoRepeatDelay := 1000 div 3;
      AutoRepeatRate := 500;
     end;
    Free;
   end;
end;


end.
